
//    (Brenkman Andrey)
//   21.02.2009  21.11.2010 18.09.2011 27.09.2011

#include <fstream>
#include <iostream>

#include "_search_switch.h"
#include "b_constants.h"
#include "move_generation.h"
#include "b_move_generation_const.h"
#include "history_heuristic.h"

#include "test_chess_bitboard.h"


uint64_t bishop_attacks_masks_database[64][512]; /// 1<<9
uint64_t rook_attacks_masks_database[64][4096]; /// 1<<12


//	===================================================================
//
void Move_generation::ini_attacks_magic_masks_database()
{
	int number_on_bit[64];
	int count_bishop_attack_squares;//       bishop_masks
	uint64_t bishop_masks;//      
	uint64_t deg_masks;
	int index;
	int count_rook_attack_squares;//       rook_masks
	uint64_t rook_masks;//      

//----------------------------------------------------
//  
	for( int square = 0; square < 64; square++ )
	{
		count_bishop_attack_squares = 0;
		bishop_masks = bishop_full_masks[square];

		//     (count_bishop_attack_squares)
		//       (number_on_bit[])
		while ( bishop_masks )
		{
			number_on_bit[count_bishop_attack_squares++] = bit_scan_forward_win32(bishop_masks);
			bishop_masks = bishop_masks & (bishop_masks - 1);
		}

		//    2      
		// ..  move_masks   :)
		//        
		uint64_t  deg = move_masks[count_bishop_attack_squares];

		for(uint64_t  i = 0; i < deg; i++ )
		{
			//      (   )
			//            
			deg_masks = ini_deg_masks(number_on_bit,count_bishop_attack_squares,i);

			index = ( deg_masks * bishop_magics[square] ) >> BISHOP_BITS_SHIFT;
			bishop_attacks_masks_database[square][index] = ini_bishop_attacks_masks(square,deg_masks);
		}
	}

//----------------------------------------------------
//  
	for( int square = 0; square < 64; square++ )
	{
		count_rook_attack_squares = 0;
		rook_masks = rook_full_masks[square];

		while ( rook_masks )
		{
			number_on_bit[count_rook_attack_squares++] = bit_scan_forward_win32(rook_masks);
			rook_masks = rook_masks & (rook_masks - 1);
		}

		uint64_t  deg = move_masks[count_rook_attack_squares];

		//deg = 20;

		for( uint64_t  i = 0; i < deg; i++ )
		{
			deg_masks = ini_deg_masks(number_on_bit,count_rook_attack_squares,i);
			index = ( deg_masks * rook_magics[square] ) >> ROOK_BITS_SHIFT;
			rook_attacks_masks_database[square][index] = ini_rook_attacks_masks(square,deg_masks);

		}
	}
}

//	===================================================================
//
uint64_t Move_generation::ini_deg_masks
(
	const int number_on_bit[], 
	const int count_attack_squares, 
	const uint64_t deg
)
{
	uint64_t deg_masks = 0;

/*
        
1,           // 0 bit .. ....0001
2,           // 1 bit .. ....0010
2,           //       .. ....0011
4,           // 2 bit .. ....0100 
 ..
..      
       deg
        
     
*/

	for( int i = 0; i < count_attack_squares; i++ )
	{
		if( deg & move_masks[i] )
		{
			deg_masks = deg_masks | move_masks[ number_on_bit[i] ];
		}
	}

	return deg_masks;
}

//	===================================================================
//    
uint64_t Move_generation::ini_rook_attacks_masks
(
	const int square, 
	const uint64_t deg_masks
)
{
	uint64_t rook_attacks_masks = 0;
	uint64_t blocking_square_mask;
	int blocking_coordinate;

		//  ------------------------------------------------------------------
		blocking_square_mask = deg_masks & rook_up_masks[square];

		if (blocking_square_mask != 0)
		{ 
			blocking_coordinate = bit_scan_forward_win32(blocking_square_mask);
			rook_attacks_masks = rook_attacks_masks | (rook_up_masks[square] ^ rook_up_masks[blocking_coordinate]);
		}
		else
		{
			rook_attacks_masks = rook_attacks_masks | rook_up_masks[square];
		}

		//  -----------------------------------------------------------------
		blocking_square_mask = deg_masks & rook_left_masks[square];
		if (blocking_square_mask != 0)
		{ 
			blocking_coordinate = bit_scan_reverse_win32(blocking_square_mask);
			rook_attacks_masks = rook_attacks_masks | (rook_left_masks[square] ^ rook_left_masks[blocking_coordinate]);
		}
		else
		{
			rook_attacks_masks = rook_attacks_masks | rook_left_masks[square];
		}

		//  ------------------------------------------------------------------
		blocking_square_mask = deg_masks & rook_down_masks[square];

		if (blocking_square_mask != 0)
		{ 
			blocking_coordinate = bit_scan_reverse_win32(blocking_square_mask);
			rook_attacks_masks = rook_attacks_masks | (rook_down_masks[square] ^ rook_down_masks[blocking_coordinate]);
		}
		else
		{
			rook_attacks_masks = rook_attacks_masks |  rook_down_masks[square];
		}

		//  -----------------------------------------------------------------
		blocking_square_mask = deg_masks & rook_right_masks[square];

		if (blocking_square_mask != 0)
		{ 
			blocking_coordinate = bit_scan_forward_win32(blocking_square_mask);
			rook_attacks_masks = rook_attacks_masks | (rook_right_masks[square] ^ rook_right_masks[blocking_coordinate]);
		}
		else
		{
			rook_attacks_masks = rook_attacks_masks |  rook_right_masks[square];
		}

	return rook_attacks_masks;
}

//	===================================================================
//     
uint64_t Move_generation::ini_bishop_attacks_masks
(
	const int square, 
	const uint64_t deg_masks
)
{

	uint64_t bishop_attacks_masks = 0;
	uint64_t blocking_square_mask; //    
	int32_t blocking_coordinate; //    

	//      (  )

	//   - --------------------------------------------------------
	blocking_square_mask = deg_masks & bishop_up_right_masks[square];

	if(blocking_square_mask != 0)
	{ 
		blocking_coordinate = bit_scan_forward_win32(blocking_square_mask);
		bishop_attacks_masks = bishop_attacks_masks | ( bishop_up_right_masks[square] ^ bishop_up_right_masks[blocking_coordinate] );
	}
	else
	{
		bishop_attacks_masks = bishop_attacks_masks | bishop_up_right_masks[square];
	}

	//   - ---------------------------------------------------------
	blocking_square_mask = deg_masks & bishop_up_left_masks[square];

	if(blocking_square_mask != 0)
	{ 
		blocking_coordinate = bit_scan_forward_win32(blocking_square_mask);
		bishop_attacks_masks = bishop_attacks_masks | ( bishop_up_left_masks[square] ^ bishop_up_left_masks[blocking_coordinate] );
	}
	else
	{
		bishop_attacks_masks = bishop_attacks_masks | bishop_up_left_masks[square];
	}

	//   - ----------------------------------------------------------
	blocking_square_mask = deg_masks & bishop_down_left_masks[square];

	if(blocking_square_mask != 0)
	{ 
		blocking_coordinate = bit_scan_reverse_win32(blocking_square_mask);
		bishop_attacks_masks = bishop_attacks_masks | ( bishop_down_left_masks[square] ^ bishop_down_left_masks[blocking_coordinate] );
	}
	else
	{
		bishop_attacks_masks = bishop_attacks_masks | bishop_down_left_masks[square];
	}

	//   - ---------------------------------------------------------
	blocking_square_mask = deg_masks & bishop_down_right_masks[square];

	if (blocking_square_mask != 0)
	{ 
		blocking_coordinate = bit_scan_reverse_win32(blocking_square_mask);
		bishop_attacks_masks = bishop_attacks_masks | ( bishop_down_right_masks[square] ^ bishop_down_right_masks[blocking_coordinate] );
	}
	else
	{
		bishop_attacks_masks = bishop_attacks_masks | bishop_down_right_masks[square];
	}

	return bishop_attacks_masks;

}

//	===================================================================
//
extern uint64_t Move_generation::get_bishop_attacks_masks_magic
(
	const uint8_t & square,///   
	const uint64_t & position_all_piece ///
)
{
	int index = 
	( ( position_all_piece & bishop_full_masks[square] ) * bishop_magics[square] )
	>> BISHOP_BITS_SHIFT ;
	
	return bishop_attacks_masks_database[square][index];
}

//	===================================================================
//
extern uint64_t Move_generation::get_rook_attacks_masks_magic
(
	const uint8_t & square,///   
	const uint64_t & position_all_piece ///
)
{

	int index = 
	( ( position_all_piece & rook_full_masks[square] ) * rook_magics[square] ) 
	>> ROOK_BITS_SHIFT ;

	return rook_attacks_masks_database[square][index];
}



//	===================================================================
//    +   ( )
void Move_generation::generator_captures_white
(
	struct List & list,//   
	const struct Position & position //      
)
{
	uint64_t from_mask; //     
	uint64_t one_pawn_mask; //    
	uint64_t move_conversion_mask; //     
	uint64_t captures_conversion_mask; //     
	uint64_t captures_mask; //  
	uint64_t attack_mask;

	uint8_t p; //    
	uint8_t from; //     
	uint8_t to; //      

	//    
	p = 0;

	//  ----------------------------------------------------------------------------------
	from_mask = position.white_piece[N_PAWN];

	while (from_mask != 0)
	{
		from = bit_scan_forward_win32(from_mask);

		//   
		one_pawn_mask = from_mask & move_masks[from];
		from_mask = from_mask & (from_mask - 1);

		/* 
		    :
		line_index = 8 * rank_index + file_index
		to = 8 * position.taking_passage_y + position.taking_passage_x;

		    :
		file_index   = line_index modulo 8 = line_index % 8  = line_index & 7
		rank_index   = line_index div 8    = line_index / 8  = line_index >> 3

		  :
		line_index     ( 0  63)
		rank_index     ( 0  7)
		file_index      ( 0  7)
		*/

		//        
		if ( from / 8 == 6 )//      ..  
							//   
		{
			// 1         
			move_conversion_mask = (one_pawn_mask << 8) & (~(position.all_piece));

			//   
			captures_conversion_mask = pawn_capture_white[from] &  position.all_black_piece;

			//      
			while (move_conversion_mask != 0)
			{
				to = bit_scan_forward_win32(move_conversion_mask);
				move_conversion_mask = move_conversion_mask & (move_conversion_mask - 1);

				//   
				//    15,   14,   13,   12
				list_record_special(list,N_PAWN,TRANSFORMATION_IN_QUEEN,p,from,to,0);
				list_record_special(list,N_PAWN,TRANSFORMATION_IN_ROOK,p,from,to,0);
				list_record_special(list,N_PAWN,TRANSFORMATION_IN_BISHOP,p,from,to,0);
				list_record_special(list,N_PAWN,TRANSFORMATION_IN_KNIGHT,p,from,to,0);
 
#if KONTROL_B
				KONTROL_move(from,to,p);
#endif//KONTROL_B
			}

			//        
			while (captures_conversion_mask != 0)
			{
				to = bit_scan_forward_win32(captures_conversion_mask);
				captures_conversion_mask = captures_conversion_mask & (captures_conversion_mask - 1);

				//     
				//    25,   24,   23,   22
				list_record_special(list,N_PAWN,CAPTURE_WITH_TRANSFORMATION_IN_QUEEN,p,from,to,position.black_name_from_coord[to]);
				list_record_special(list,N_PAWN,CAPTURE_WITH_TRANSFORMATION_IN_ROOK,p,from,to,position.black_name_from_coord[to]);
				list_record_special(list,N_PAWN,CAPTURE_WITH_TRANSFORMATION_IN_BISHOP,p,from,to,position.black_name_from_coord[to]);
				list_record_special(list,N_PAWN,CAPTURE_WITH_TRANSFORMATION_IN_KNIGHT,p,from,to,position.black_name_from_coord[to]);

#if KONTROL_B
				KONTROL_captures_W(position,from,to,p,120);
#endif//KONTROL_B
			}

		}
		else
		{
			//   
			captures_mask = pawn_capture_white[from] &  position.all_black_piece;

			while (captures_mask != 0)
			{
				to = bit_scan_forward_win32(captures_mask);
				list_capture_white(list,N_PAWN,CAPTURE_MOVE,from,to,p,position);
				captures_mask = captures_mask & (captures_mask - 1);
			}
		}

		// 4    
		if (position.taking_passage_yes == 1)
		{
			//           
			//        4    
			//        .. 
			//     
			if ( from / 8 == 4 )
			{
				to = 8 * position.taking_passage_y + position.taking_passage_x;

				//    .        
				if (((from - to) == -9) && (from % 8 != 7))
				{
					list_record_special(list,N_PAWN,TAKING_PASSAGE,p,from,to,position.black_name_from_coord[to - 8]);

#if KONTROL_B
					to = to - 8;
					KONTROL_captures_W(position,from,to,p,130);
#endif//KONTROL_B
				}

				//    .        
				if (((from - to) == -7) && (from % 8 != 0))
				{
					list_record_special(list,N_PAWN,TAKING_PASSAGE,p,from,to,position.black_name_from_coord[to - 8]);

#if KONTROL_B
					to = to - 8;
					KONTROL_captures_W(position,from,to,p,130);
#endif//KONTROL_B
				}
			}
		}
	}
	// ----------------------------------------------------------------------------------


	//   ----------------------------------------------------------------------------------
	//               
	//         
	from_mask = position.white_piece[N_KNIGHT];

	while (from_mask != 0)
	{
		from = bit_scan_forward_win32(from_mask);
		from_mask = from_mask & (from_mask - 1);
		captures_mask = (position.all_black_piece & knigt_moves_masks[from]);

		while (captures_mask != 0)
		{
			to = bit_scan_forward_win32(captures_mask);
			list_capture_white(list,N_KNIGHT,CAPTURE_MOVE,from,to,p,position);
			captures_mask = captures_mask & (captures_mask - 1);
		}
	}
	//  ----------------------------------------------------------------------------------


	//  ---------------------------------------------------------------------------------- 
	from_mask = position.white_piece[N_BISHOP];

	while (from_mask != 0)
	{
		from = bit_scan_forward_win32(from_mask);
		from_mask = from_mask & (from_mask - 1);


		attack_mask = get_bishop_attacks_masks_magic(from,position.all_piece);
		captures_mask = attack_mask & position.all_black_piece;

		while (captures_mask != 0)
		{
			to = bit_scan_forward_win32(captures_mask);
			captures_mask = captures_mask & (captures_mask - 1);
			list_capture_white(list,N_BISHOP,CAPTURE_MOVE,from,to,p,position);
		}

	}
	// ---------------------------------------------------------------------------------- 


	//  ---------------------------------------------------------------------------------
	from_mask = position.white_piece[N_ROOK];

	while (from_mask != 0)
	{
		from = bit_scan_forward_win32(from_mask);
		from_mask = from_mask & (from_mask - 1);

		attack_mask = get_rook_attacks_masks_magic(from,position.all_piece);
		captures_mask = attack_mask & position.all_black_piece;

		while (captures_mask != 0)
		{
			to = bit_scan_forward_win32(captures_mask);
			captures_mask = captures_mask & (captures_mask - 1);
			list_capture_white(list,N_ROOK,CAPTURE_MOVE,from,to,p,position);
		}

	}
	// ----------------------------------------------------------------------------------


	//  ---------------------------------------------------------------------------------
	//    
	from_mask = position.white_piece[N_QUEEN];

	while (from_mask != 0)
	{
		from = bit_scan_forward_win32(from_mask);
		from_mask = from_mask & (from_mask - 1);


		captures_mask = get_rook_attacks_masks_magic(from,position.all_piece);
		attack_mask = get_bishop_attacks_masks_magic(from,position.all_piece);
		captures_mask = (captures_mask | attack_mask) & position.all_black_piece;

		while (captures_mask != 0)
		{
			to = bit_scan_forward_win32(captures_mask);
			captures_mask = captures_mask & (captures_mask - 1);
			list_capture_white(list,N_QUEEN,CAPTURE_MOVE,from,to,p,position);
		}
	}
	// --------------------------------------------------------------------------------------


	//  --------------------------------------------------------------------------------
	//    
	from_mask = position.white_piece[N_KING];

	if (from_mask != 0)
	{
		//   ( (    )  )
		from = bit_scan_forward_win32(from_mask);

		//    
		captures_mask = (position.all_black_piece & king_moves_masks[from]);

		while (captures_mask != 0)
		{
			//   
			to = bit_scan_forward_win32(captures_mask);
			list_capture_white(list,N_KING,CAPTURE_MOVE,from,to,p,position);
			captures_mask = captures_mask & (captures_mask - 1);

		}

	}
	//  --------------------------------------------------------------------------------

	list.end_list = p;
	list.end_captures = p;

}

//	===================================================================
//     +   ( )
void Move_generation::generator_captures_black
(
	struct List & list,//   
	const struct Position & position //     
)
{
	uint64_t from_mask; //     
	uint64_t one_pawn_mask; //    
	uint64_t move_conversion_mask; //     
	uint64_t captures_conversion_mask; //     
	uint64_t captures_mask; //  
	uint64_t attack_mask;

	uint8_t p; //    
	uint8_t from; //     
	uint8_t to; //     

	//    
	p = 0;

	//  ----------------------------------------------------------------------------------
	from_mask = position.black_piece[N_PAWN];

	while (from_mask != 0)
	{
		from = bit_scan_forward_win32(from_mask);

		//    
		one_pawn_mask = from_mask & move_masks[from];
		from_mask = from_mask & (from_mask - 1);

		if ( from / 8 == 1 )
		{
			//        
			// 1         
			move_conversion_mask = (one_pawn_mask >> 8) & (~(position.all_piece));

			//   
			captures_conversion_mask = pawn_capture_black[from] &  position.all_white_piece;

			//      
			while (move_conversion_mask != 0)
			{
				to = bit_scan_forward_win32(move_conversion_mask);
				move_conversion_mask = move_conversion_mask & (move_conversion_mask - 1);

				//   
				//    15,   14,   13,
				//    12
				list_record_special(list,N_PAWN,TRANSFORMATION_IN_QUEEN,p,from,to,0);
				list_record_special(list,N_PAWN,TRANSFORMATION_IN_ROOK,p,from,to,0);
				list_record_special(list,N_PAWN,TRANSFORMATION_IN_BISHOP,p,from,to,0);
				list_record_special(list,N_PAWN,TRANSFORMATION_IN_KNIGHT,p,from,to,0);
 
#if KONTROL_B
				KONTROL_move(from,to,p);
#endif//KONTROL_B
			}

			//        
			//        
			while (captures_conversion_mask != 0)
			{
				to = bit_scan_forward_win32(captures_conversion_mask);
				captures_conversion_mask = captures_conversion_mask & (captures_conversion_mask - 1);

				//     
				//    25,   24,   23,
				//    22
				list_record_special(list,N_PAWN,CAPTURE_WITH_TRANSFORMATION_IN_QUEEN,p,from,to,position.white_name_from_coord[to]);
				list_record_special(list,N_PAWN,CAPTURE_WITH_TRANSFORMATION_IN_ROOK,p,from,to,position.white_name_from_coord[to]);
				list_record_special(list,N_PAWN,CAPTURE_WITH_TRANSFORMATION_IN_BISHOP,p,from,to,position.white_name_from_coord[to]);
				list_record_special(list,N_PAWN,CAPTURE_WITH_TRANSFORMATION_IN_KNIGHT,p,from,to,position.white_name_from_coord[to]);

#if KONTROL_B
				KONTROL_captures_B(position,from,to,p,120);
#endif//KONTROL_B
			}
		}
		else
		{
			//   
			captures_mask =  pawn_capture_black[from] &  position.all_white_piece;

			while (captures_mask != 0)
			{
				to = bit_scan_forward_win32(captures_mask);
				list_capture_black(list,N_PAWN,CAPTURE_MOVE,from,to,p,position);
				captures_mask = captures_mask & (captures_mask - 1);
			}
		}

		// 4    
		if (position.taking_passage_yes == 1)
		{
			if ( from / 8 == 3 )
			{
				to = 8 * position.taking_passage_y + position.taking_passage_x;

				if (((from - to) == 9) && (from % 8 != 0))
				{
					list_record_special(list,N_PAWN,TAKING_PASSAGE,p,from,to,position.white_name_from_coord[to + 8]);

#if KONTROL_B
					to = to + 8;
					KONTROL_captures_B(position,from,to,p,130);
#endif//KONTROL_B
				}

				if (((from - to) == 7) && (from % 8 != 7))
				{
					list_record_special(list,N_PAWN,TAKING_PASSAGE,p,from,to,position.white_name_from_coord[to + 8]);

#if KONTROL_B
					to = to + 8;
					KONTROL_captures_B(position,from,to,p,130);
#endif//KONTROL_B
				}
			}
		}
	}
	// ----------------------------------------------------------------------------------


	//   ----------------------------------------------------------------------------------
	//               
	//         
	//  
	from_mask = position.black_piece[N_KNIGHT];

	while (from_mask != 0)
	{
		from = bit_scan_forward_win32(from_mask);
		from_mask = from_mask & (from_mask - 1);
		captures_mask = (position.all_white_piece & knigt_moves_masks[from]);

		while (captures_mask != 0)
		{
		   to = bit_scan_forward_win32(captures_mask);
		   list_capture_black(list,N_KNIGHT,CAPTURE_MOVE,from,to,p,position);
		   captures_mask = captures_mask & (captures_mask - 1);
		}
	}
	//  ----------------------------------------------------------------------------------

	//  ---------------------------------------------------------------------------------- 
	from_mask = position.black_piece[N_BISHOP];

	while (from_mask != 0)
	{
		from = bit_scan_forward_win32(from_mask);
		from_mask = from_mask & (from_mask - 1);


		attack_mask = get_bishop_attacks_masks_magic(from,position.all_piece);
		captures_mask = attack_mask & position.all_white_piece;

		while (captures_mask != 0)
		{
			to = bit_scan_forward_win32(captures_mask);
			captures_mask = captures_mask & (captures_mask - 1);
			list_capture_black(list,N_BISHOP,CAPTURE_MOVE,from,to,p,position);
		}

	}
	// ---------------------------------------------------------------------------------- 


	//  ---------------------------------------------------------------------------------
	from_mask = position.black_piece[N_ROOK];

	while (from_mask != 0)
	{
		from = bit_scan_forward_win32(from_mask);
		from_mask = from_mask & (from_mask - 1);

		attack_mask = get_rook_attacks_masks_magic(from,position.all_piece);
		captures_mask = attack_mask & position.all_white_piece;

		while (captures_mask != 0)
		{
			to = bit_scan_forward_win32(captures_mask);
			captures_mask = captures_mask & (captures_mask - 1);
			list_capture_black(list,N_ROOK,CAPTURE_MOVE,from,to,p,position);
		}

	}
	// ----------------------------------------------------------------------------------


	//  ---------------------------------------------------------------------------------
	from_mask = position.black_piece[N_QUEEN];

	while (from_mask != 0)
	{
		from = bit_scan_forward_win32(from_mask);
		from_mask = from_mask & (from_mask - 1);


		captures_mask = get_rook_attacks_masks_magic(from,position.all_piece);
		attack_mask = get_bishop_attacks_masks_magic(from,position.all_piece);
		captures_mask = (captures_mask | attack_mask) & position.all_white_piece;

		while (captures_mask != 0)
		{
			to = bit_scan_forward_win32(captures_mask);
			captures_mask = captures_mask & (captures_mask - 1);
			list_capture_black(list,N_QUEEN,CAPTURE_MOVE,from,to,p,position);
		}
	}
	// --------------------------------------------------------------------------------------

	//  ---------------------------------------------------------------------------------
	from_mask = position.black_piece[N_KING];

	if (from_mask != 0)
	{
		from = bit_scan_forward_win32(from_mask);
		captures_mask = (position.all_white_piece & king_moves_masks[from]);

		while (captures_mask != 0)
		{
			to = bit_scan_forward_win32(captures_mask);
			list_capture_black(list,N_KING,CAPTURE_MOVE,from,to,p,position);
			captures_mask = captures_mask & (captures_mask - 1);
		}
	}
	// ---------------------------------------------------------------------------------


	list.end_list = p;
	list.end_captures = p;

}

//	===================================================================
//     (..     +  )
//( )
void Move_generation::generator_move_white
(
	struct List & list,//   
	const struct Position & position //     
)
{
	uint64_t from_mask; //     
	uint64_t move_mask; // 
	uint64_t one_pawn_mask; //    
	uint64_t move_mask_d; //      

	uint8_t p; //    
	uint8_t from; //     
	uint8_t to; //     

	p = list.end_list;

	//    
	//  --------------------------------------------------------------------------------
	//    
	from_mask = position.white_piece[N_KING];

	if (from_mask != 0)
	{
		//   ( (    ) 
		//  )
		from = bit_scan_forward_win32(from_mask);

		//    ( )
		move_mask = king_moves_masks[from] & (~position.all_piece);

		while (move_mask != 0)
		{
			to = bit_scan_forward_win32(move_mask);
			move_mask = move_mask & (move_mask - 1);
			list_record_move_white(list,N_KING,SIMPLE_MOVE,from,to,p,position);
		}

		//   

		//    
		if (position.castling_Q)
		{
			// 
			to = C1;
#if KONTROL_B
			KONTROL_move(from,to,p);
#endif//KONTROL_B

			//   1 2 3     
			if (((~position.all_piece) & CASTLING_Q ) == CASTLING_Q)
			{
				//     0-0-0      2 3 4 
				if ((white_check_square(C1,position) == 0) 
				&& (white_check_square(D1,position) == 0) 
				&& (white_check_square(E1,position) == 0))
				{
					list_record_special(list,N_KING,LONG_CASTLING,p,from,to,0);
				}
			}
		}

		//        
		if (position.castling_K)
		{
			to = G1;
#if KONTROL_B
			KONTROL_move(from,to,p);
#endif//KONTROL_B

			//  5 6     
			if (((~position.all_piece) & CASTLING_K ) == CASTLING_K)
			{
				//    0-0       4 5 6
				if ((white_check_square(E1,position) == 0)
				&& (white_check_square(F1,position) == 0)
				&& (white_check_square(G1,position) == 0))
				{
					list_record_special(list,N_KING,SHORT_CASTLING,p,from,to,0);
				}
			}
		}
	}

	//  ----------------------------------------------------------------------------------
	//    
	from_mask = position.white_piece[N_QUEEN];

	while (from_mask != 0)
	{
		from = bit_scan_forward_win32(from_mask);

		//      b = b & (b-1)  
		//      
		from_mask = from_mask & (from_mask - 1);

		uint64_t mobility_mask = 0; //  

		move_mask = get_rook_attacks_masks_magic(from,position.all_piece);
		//move_mask = move_mask ^ (position.all_piece & move_mask);
		move_mask = move_mask & (~position.all_piece);

		mobility_mask = get_bishop_attacks_masks_magic(from,position.all_piece);
		//mobility_mask = mobility_mask ^ (position.all_piece & mobility_mask);
		mobility_mask = mobility_mask & (~position.all_piece);

		move_mask = move_mask |	mobility_mask;

		while (move_mask != 0)
		{
			to = bit_scan_forward_win32(move_mask);
			move_mask = move_mask & (move_mask - 1);
			list_record_move_white(list,N_QUEEN,SIMPLE_MOVE,from,to,p,position);
		}
	}

	//  ----------------------------------------------------------------------------------
	from_mask = position.white_piece[N_ROOK];

	while (from_mask != 0)
	{
		from = bit_scan_forward_win32(from_mask);
		from_mask = from_mask & (from_mask - 1);

		move_mask = get_rook_attacks_masks_magic(from,position.all_piece);
		//move_mask = move_mask ^ (position.all_piece & move_mask);
		move_mask = move_mask & (~position.all_piece);

		while (move_mask != 0)
		{
			to = bit_scan_forward_win32(move_mask);
			move_mask = move_mask & (move_mask - 1);
			list_record_move_white(list,N_ROOK,SIMPLE_MOVE,from,to,p,position);
		}
	}

	//  ---------------------------------------------------------------------------------- 
	from_mask = position.white_piece[N_BISHOP];

	while (from_mask != 0)
	{
		from = bit_scan_forward_win32(from_mask);
		from_mask = from_mask & (from_mask - 1);

		move_mask = get_bishop_attacks_masks_magic(from,position.all_piece);
		//move_mask = move_mask ^ (position.all_piece & move_mask);
		move_mask = move_mask & (~position.all_piece);

		while (move_mask != 0)
		{
			to = bit_scan_forward_win32(move_mask);
			move_mask = move_mask & (move_mask - 1);
			list_record_move_white(list,N_BISHOP,SIMPLE_MOVE,from,to,p,position);
		}

	}

	//   ----------------------------------------------------------------------------------
	//               
	//         
	from_mask = position.white_piece[N_KNIGHT];

	while (from_mask != 0)
	{
		from = bit_scan_forward_win32(from_mask);
		from_mask = from_mask & (from_mask - 1);

		move_mask = knigt_moves_masks[from] & (~position.all_piece); //  

		while (move_mask != 0)
		{
			to = bit_scan_forward_win32(move_mask);
			move_mask = move_mask & (move_mask - 1);
			list_record_move_white(list,N_KNIGHT,SIMPLE_MOVE,from,to,p,position);
		}
	}

	//  ----------------------------------------------------------------------------------
	from_mask = position.white_piece[N_PAWN];

	while (from_mask != 0)
	{
		from = bit_scan_forward_win32(from_mask);

		//   
		one_pawn_mask = from_mask & move_masks[from];
		from_mask = from_mask & (from_mask - 1);

		// 1         
		move_mask = (one_pawn_mask << 8) & (~(position.all_piece));

		// 2     2   
		//        2 
		move_mask_d = one_pawn_mask & LINE_1;

		//       
		move_mask_d = (move_mask_d << 8) & (~(position.all_piece));

		//    
		move_mask_d = (move_mask_d << 8) & (~(position.all_piece));

		//            
		move_mask = (move_mask | move_mask_d) ;

		//   
		move_mask = move_mask & ~LINE_7;

		while (move_mask != 0)
		{
			to = bit_scan_forward_win32(move_mask);
			move_mask = move_mask & (move_mask - 1);
			list_record_move_white(list,N_PAWN,SIMPLE_MOVE,from,to,p,position);
		}
	}

	list.end_list = p;

}

//	===================================================================
//     (..     +  )( )
void Move_generation::generator_move_black
(
	struct List & list,//   
	const struct Position & position //     
)
{
	uint64_t from_mask; //     
	uint64_t move_mask; //         
	uint64_t one_pawn_mask; //    
	uint64_t move_mask_d; //      

	uint8_t p; //    
	uint8_t from; //     
	uint8_t to; //     

	p = list.end_list;

	//    
	//  ---------------------------------------------------------------------------------
	from_mask = position.black_piece[N_KING];

	if (from_mask != 0)
	{
		from = bit_scan_forward_win32(from_mask);

		//    ( )
		move_mask = king_moves_masks[from] & (~position.all_piece);

		while (move_mask != 0)
		{
			to = bit_scan_forward_win32(move_mask);
			move_mask = move_mask & (move_mask - 1);
			list_record_move_black(list,N_KING,SIMPLE_MOVE,from,to,p,position);
		}

		//   

		//    
		if (position.castling_q)
		{
			to = C8;
#if KONTROL_B
			KONTROL_move(from,to,p);
#endif//KONTROL_B

			//    1 2 3     
			if ((~(position.all_piece) & CASTLING_q )== CASTLING_q)
			{
				//     0-0-0 58 59 60
				if ((black_check_square(C8,position) == 0)
				&& (black_check_square(D8,position) == 0)
				&& (black_check_square(E8,position) == 0))
				{
					list_record_special(list,N_KING,LONG_CASTLING,p,from,to,0);			 
				}
			}
		}

		//        
		if (position.castling_k)
		{
			to = G8;
#if KONTROL_B
			KONTROL_move(from,to,p);
#endif//KONTROL_B

			//    5 6     
			if ((~(position.all_piece) & CASTLING_k )== CASTLING_k)
			{
				//     0-0 60 61 62
				if ((black_check_square(E8,position) == 0)
				&& (black_check_square(F8,position) == 0)
				&&(black_check_square(G8,position) == 0))
				{
					list_record_special(list,N_KING,SHORT_CASTLING,p,from,to,0);
				}
			}
		}
	}

	//  ----------------------------------------------------------------------------------
	//   
	from_mask = position.black_piece[N_QUEEN];

	while (from_mask != 0)
	{
		from = bit_scan_forward_win32(from_mask);

		//      b = b & (b-1)   
		//    
		from_mask = from_mask & (from_mask - 1);

		uint64_t mobility_mask = 0; //  

		move_mask = get_rook_attacks_masks_magic(from,position.all_piece);
		//move_mask = move_mask ^ (position.all_piece & move_mask);
		move_mask = move_mask & (~position.all_piece);

		mobility_mask = get_bishop_attacks_masks_magic(from,position.all_piece);
		//mobility_mask = mobility_mask ^ (position.all_piece & mobility_mask);
		mobility_mask = mobility_mask & (~position.all_piece);

		move_mask = move_mask |	mobility_mask;


		while (move_mask != 0)
		{
			to = bit_scan_forward_win32(move_mask);
			move_mask = move_mask & (move_mask - 1);
			list_record_move_black(list,N_QUEEN,SIMPLE_MOVE,from,to,p,position);
		}
	}

	//  ----------------------------------------------------------------------------------
	from_mask = position.black_piece[N_ROOK];

	while (from_mask != 0)
	{
		from = bit_scan_forward_win32(from_mask);
		from_mask = from_mask & (from_mask - 1);

		move_mask = get_rook_attacks_masks_magic(from,position.all_piece);
		//move_mask = move_mask ^ (position.all_piece & move_mask);
		move_mask = move_mask & (~position.all_piece);

		while (move_mask != 0)
		{
			to = bit_scan_forward_win32(move_mask);
			move_mask = move_mask & (move_mask - 1);
			list_record_move_black(list,N_ROOK,SIMPLE_MOVE,from,to,p,position);
		}
	}

	//  ---------------------------------------------------------------------------------- 
	from_mask = position.black_piece[N_BISHOP];

	while (from_mask != 0)
	{
		from = bit_scan_forward_win32(from_mask);
		from_mask = from_mask & (from_mask - 1);

		move_mask = get_bishop_attacks_masks_magic(from,position.all_piece);
		//move_mask = move_mask ^ (position.all_piece & move_mask);
		move_mask = move_mask & (~position.all_piece);

		while (move_mask != 0)
		{
			to = bit_scan_forward_win32(move_mask);
			move_mask = move_mask & (move_mask - 1);
			list_record_move_black(list,N_BISHOP,SIMPLE_MOVE,from,to,p,position);
		}
	}

	//   ----------------------------------------------------------------------------------
	//               
	//         
	from_mask = position.black_piece[N_KNIGHT];

	while (from_mask != 0)
	{
		from = bit_scan_forward_win32(from_mask);
		from_mask = from_mask & (from_mask - 1);

		move_mask = knigt_moves_masks[from] & (~position.all_piece); //  

		while (move_mask != 0)
		{
			to = bit_scan_forward_win32(move_mask);
			move_mask = move_mask & (move_mask - 1);
			list_record_move_black(list,N_KNIGHT,SIMPLE_MOVE,from,to,p,position);
		}
	}

	//  ----------------------------------------------------------------------------------
	from_mask = position.black_piece[N_PAWN];

	while (from_mask != 0)
	{
		from = bit_scan_forward_win32(from_mask);
		one_pawn_mask = from_mask & move_masks[from];
		from_mask = from_mask & (from_mask - 1);

		// 1         
		move_mask = (one_pawn_mask >> 8) & (~(position.all_piece));

		// 2     2   
		//        7 
		move_mask_d = one_pawn_mask & LINE_6;

		//       
		move_mask_d = (move_mask_d >> 8) & (~(position.all_piece));

		//    
		move_mask_d = (move_mask_d >> 8) & (~(position.all_piece));

		//            
		move_mask = (move_mask | move_mask_d) ;

		//   
		move_mask = move_mask & ~LINE_0;

		while (move_mask != 0)
		{
			to = bit_scan_forward_win32(move_mask);
			move_mask = move_mask & (move_mask - 1);
			list_record_move_black(list,N_PAWN,SIMPLE_MOVE,from,to,p,position);
		}
	}

	list.end_list = p;

}

//	===================================================================
//           0    
uint8_t Move_generation::king_white_check
(
	const struct Position & position //     
)
{

	uint64_t from_mask; //     
	uint64_t attack_mask; //   
	uint64_t capture_mask; //  


	uint8_t from; //     

	//    
	from_mask = position.white_piece[N_KING];

	//   ( (    )  )
	from = bit_scan_forward_win32(from_mask);

	//   
    attack_mask = get_rook_attacks_masks_magic(from,position.all_piece);

	//         
	capture_mask = attack_mask & position.black_piece[N_ROOK];
	if (capture_mask != 0) return N_ROOK;

	//         
	capture_mask = attack_mask & position.black_piece[N_QUEEN];
	if (capture_mask != 0) return N_QUEEN;

	//   
    attack_mask = get_bishop_attacks_masks_magic(from,position.all_piece);

	//          
	capture_mask = attack_mask & position.black_piece[N_BISHOP];
	if (capture_mask != 0) return N_BISHOP;

	//         
	capture_mask = attack_mask & position.black_piece[N_QUEEN];
	if (capture_mask != 0) return N_QUEEN;


	//   
	capture_mask = (knigt_moves_masks[from] & position.black_piece[N_KNIGHT]);
	if (capture_mask != 0) return N_KNIGHT;


	//   
	capture_mask = (king_moves_masks[from] & position.black_piece[N_KING]);
	if (capture_mask != 0) return N_KING;

//   
	if ((from / 7) < 8)
	{
		if (from % 8 != 0)
		{
			if (position.black_name_from_coord[from + 7] == N_PAWN)
			{
				return N_PAWN;
			}
		}

		if (from % 8 != 7)
		{
			if (position.black_name_from_coord[from + 9] == N_PAWN)
			{
				return N_PAWN;
			}
		}
	}

	return 0; 
}


//	===================================================================
//           0    
uint8_t Move_generation::king_black_check
(
	const struct Position & position //     
)
{

	uint64_t from_mask; //     
	uint64_t attack_mask; //   
	uint64_t capture_mask; //  


	uint8_t from; //     

	//    
	from_mask = position.black_piece[N_KING];

	//   ( (    )  )
	from = bit_scan_forward_win32(from_mask);

	//   
    attack_mask = get_rook_attacks_masks_magic(from,position.all_piece);

	//         
	capture_mask = attack_mask & position.white_piece[N_ROOK];
	if (capture_mask != 0) return N_ROOK;

	//         
	capture_mask = attack_mask & position.white_piece[N_QUEEN];
	if (capture_mask != 0) return N_QUEEN;

	//   
    attack_mask = get_bishop_attacks_masks_magic(from,position.all_piece);

	//          
	capture_mask = attack_mask & position.white_piece[N_BISHOP];
	if (capture_mask != 0) return N_BISHOP;

	//         
	capture_mask = attack_mask & position.white_piece[N_QUEEN];
	if (capture_mask != 0) return N_QUEEN;


	//   
	capture_mask = (knigt_moves_masks[from] & position.white_piece[N_KNIGHT]);
	if (capture_mask != 0) return N_KNIGHT;


	//   
	capture_mask = (king_moves_masks[from] & position.white_piece[N_KING]);
	if (capture_mask != 0) return N_KING;

	//   
	if ((from / 7) > 1 )
	{
		if (from % 8 != 7)
		{
			if (position.white_name_from_coord[from - 7] == N_PAWN)
			{
				return N_PAWN;
			}
		}
 
		if (from % 8 != 0)
		{
			if (position.white_name_from_coord[from - 9] == N_PAWN)
			{
				return N_PAWN;
			}
		}
	}

	return 0;
 
}


//private

//	===================================================================
//            0
//    
inline uint8_t Move_generation::white_check_square
(
	const uint8_t & sq,//  (0-63) 
	const struct Position & position //     
)
{
	uint64_t attack_mask; //   
	uint64_t capture_mask; //  

	//   
    attack_mask = get_rook_attacks_masks_magic(sq,position.all_piece);

	//         
	capture_mask = attack_mask & position.black_piece[N_ROOK];
	if (capture_mask != 0) return N_ROOK;

	//         
	capture_mask = attack_mask & position.black_piece[N_QUEEN];
	if (capture_mask != 0) return N_QUEEN;

	//   
    attack_mask = get_bishop_attacks_masks_magic(sq,position.all_piece);

	//          
	capture_mask = attack_mask & position.black_piece[N_BISHOP];
	if (capture_mask != 0) return N_BISHOP;

	//         
	capture_mask = attack_mask & position.black_piece[N_QUEEN];
	if (capture_mask != 0) return N_QUEEN;


	//   
	capture_mask = (knigt_moves_masks[sq] & position.black_piece[N_KNIGHT]);
	if (capture_mask != 0) return N_KNIGHT;


	//   
	capture_mask = (king_moves_masks[sq] & position.black_piece[N_KING]);
	if (capture_mask != 0) return N_KING;

//   
	if ((sq / 7) < 8)
	{
		if (sq % 8 != 0)
		{
			if (position.black_name_from_coord[sq + 7] == N_PAWN)
			{
				return N_PAWN;
			}
		}

		if (sq % 8 != 7)
		{
			if (position.black_name_from_coord[sq + 9] == N_PAWN)
			{
				return N_PAWN;
			}
		}
	}

	return 0; 

}

//	===================================================================
//          
//    0    
inline uint8_t Move_generation::black_check_square
(
	const uint8_t & sq,//  (0-63) 
	const struct Position & position //     
)
{
	uint64_t attack_mask; //   
	uint64_t capture_mask; //  

	//   
    attack_mask = get_rook_attacks_masks_magic(sq,position.all_piece);

	//         
	capture_mask = attack_mask & position.white_piece[N_ROOK];
	if (capture_mask != 0) return N_ROOK;

	//         
	capture_mask = attack_mask & position.white_piece[N_QUEEN];
	if (capture_mask != 0) return N_QUEEN;

	//   
    attack_mask = get_bishop_attacks_masks_magic(sq,position.all_piece);

	//          
	capture_mask = attack_mask & position.white_piece[N_BISHOP];
	if (capture_mask != 0) return N_BISHOP;

	//         
	capture_mask = attack_mask & position.white_piece[N_QUEEN];
	if (capture_mask != 0) return N_QUEEN;


	//   
	capture_mask = (knigt_moves_masks[sq] & position.white_piece[N_KNIGHT]);
	if (capture_mask != 0) return N_KNIGHT;


	//   
	capture_mask = (king_moves_masks[sq] & position.white_piece[N_KING]);
	if (capture_mask != 0) return N_KING;

	//   
	if ((sq / 7) > 1 )
	{
		if (sq % 8 != 7)
		{
			if (position.white_name_from_coord[sq - 7] == N_PAWN)
			{
				return N_PAWN;
			}
		}
 
		if (sq % 8 != 0)
		{
			if (position.white_name_from_coord[sq - 9] == N_PAWN)
			{
				return N_PAWN;
			}
		}
	}

	return 0; 
}

//	===================================================================
//    -   
//        ( )
inline void Move_generation::list_capture_white
(
	struct List & list,//   
	const uint8_t name,//   
	const uint8_t type_move,//  
	const uint8_t & from,//  
	const uint8_t & to,//  
	uint8_t & count,//    
	const struct Position & position //     
)
{
	uint8_t capture;//  

#if KONTROL_B
	KONTROL_captures_W(position,from,to,count,name);
#endif//KONTROL_B

	//  
	capture = position.black_name_from_coord[to];

	//   (mvv/lva   )   
	list.sorting_score[count] = 10 * capture - name; 

	// 6  
	//     (1 -  2 -  3 -  4 -  5 -  
	// 6 - )
	list.move[count] = capture

	// 6  
	//    (0 -   1    2   3   
	//4    5    
	//12 -    13 -    14 -    
	//15 -    
	//22 -      23 -      
	//24 -      
	//25 -      )
	| (type_move << 6)

	// 6  
	//    (0 -63)
	| (from << 12)

	// 6  
	//    (0 -63)
	| (to << 18)

	// 3  
	//   (0 - 6)
	| (name << 24);

	//-------------------------------------------------------------------------------
	//std::cout << "move[count]= "<< move[count] << std::endl;
	//
	//std::cout << "position.black_name_from_coord[to]= "<< position.black_name_from_coord[to] << std::endl;
	//std::cout << "position.black_name_from_coord[to]= "<< ( move[p] & 63) << std::endl;
	//
	//std::cout << "type_move= "<< type_move << std::endl;
	//std::cout << "type_move= "<< ((move[count]>>6)& 63)  << std::endl;
	//
	//std::cout << "from= "<< from << std::endl;
	//std::cout << "from= "<< ((move[count]>>12)& 63)  << std::endl;
	//
	//std::cout << "to= "<< to << std::endl;
	//std::cout << "to= "<< ((move[count]>>18)& 63)  << std::endl;
	//
	//std::cout << "name= "<< name << std::endl;
	//std::cout << "name= "<<  ((move[count]>>24)& 7)  << std::endl;
	//-------------------------------------------------------------------------------

	count = count + 1;
}

//	===================================================================
//    -   
//        ( )
inline void Move_generation::list_capture_black
(
	struct List & list,//   
	const uint8_t name,//   
	const uint8_t type_move,//  
	const uint8_t & from,//  
	const uint8_t & to,//  
	uint8_t & count,//    
	const struct Position & position //     
)
{
	uint8_t capture;//  

#if KONTROL_B
	KONTROL_captures_B(position,from,to,count,name);
#endif//KONTROL_B

	//  
	capture = position.white_name_from_coord[to];

	//   (mvv/lva   )   
	list.sorting_score[count] = 10 * capture - name; 

	// 6  
	//     (1 -  2 -  3 -  4 -  5 -  6 - )
	list.move[count] = capture

	// 6  
	//    (0 -   1    2   3   
	//4    5    
	//12 -    13 -    14 -    15 -    
	//22 -      23 -      24 -      
	//25 -      )
	| (type_move << 6)

	// 6  
	//    (0 -63)
	| (from << 12)

	// 6  
	//    (0 -63)
	| (to << 18)

	// 3  
	//   (0 - 6)
	| (name << 24);

	//-------------------------------------------------------------------------------
	//std::cout << "move[count]= "<< move[count] << std::endl;
	//
	//std::cout << "position.white_name_from_coord[to]= "<< position.white_name_from_coord[to] << std::endl;
	//std::cout << "position.white_name_from_coord[to]= "<< ( move[p] & 63) << std::endl;
	//
	//std::cout << "type_move= "<< type_move << std::endl;
	//std::cout << "type_move= "<< ((move[count]>>6)& 63)  << std::endl;
	//
	//std::cout << "from= "<< from << std::endl;
	//std::cout << "from= "<< ((move[count]>>12)& 63)  << std::endl;
	//
	//std::cout << "to= "<< to << std::endl;
	//std::cout << "to= "<< ((move[count]>>18)& 63)  << std::endl;
	//
	//std::cout << "name= "<< name << std::endl;
	//std::cout << "name= "<<  ((move[count]>>24)& 7)  << std::endl;
	//-------------------------------------------------------------------------------

	count = count + 1;
}

//	===================================================================
//   
inline void Move_generation::list_record_move_white
(
	struct List & list,//   
	const uint8_t name,//   
	const uint8_t type_move,//  
	const uint8_t & from,//  
	const uint8_t & to,//  
	uint8_t & count,//    
	const struct Position & position //     
)
{

#if KONTROL_B
	KONTROL_move(from,to,count);
#endif//KONTROL_B

#if SW_HISTORY
	//   
	list.sorting_score[count] = History_heuristic::get_white_history(name,to); 
#endif//#if SW_HISTORY

	// 6  
	//   0
	list.move[count] = 0

	// 6  
	//    (0 -   1    2   3   
	//4    5    
	//12 -    13 -    14 -    15 -    
	//22 -      23 -      24 -      
	//25 -      )
	| (type_move << 6)

	// 6  
	//    (0 -63)
	| (from << 12)

	// 6  
	//    (0 -63)
	| (to << 18)

	// 3  
	//   (0 - 6)
	| (name << 24);

	//-------------------------------------------------------------------------------
	//std::cout << "move[p]= "<< move[p] << std::endl;
	//
	//std::cout << "0= "<< 0 << std::endl;
	//std::cout << "0= "<< ( move[p] & 63) << std::endl;
	//
	//std::cout << "d_m= "<< d_m << std::endl;
	//std::cout << "d_m= "<< ((move[p]>>6)& 63)  << std::endl;
	//
	//std::cout << "from= "<< from << std::endl;
	//std::cout << "from= "<< ((move[p]>>12)& 63)  << std::endl;
	//
	//std::cout << "to= "<< to << std::endl;
	//std::cout << "to= "<< ((move[p]>>18)& 63)  << std::endl;
	//
	//std::cout << "name= "<< name << std::endl;
	//std::cout << "name= "<<  ((move[p]>>24)& 7)  << std::endl;

	//-------------------------------------------------------------------------------

	count = count + 1;
}

//	===================================================================
//   
inline void Move_generation::list_record_move_black
(
	struct List & list,//   
	const uint8_t name,//   
	const uint8_t type_move,//  
	const uint8_t & from,//  
	const uint8_t & to,//  
	uint8_t & count,//    
	const struct Position & position //     
)
{

#if KONTROL_B
	KONTROL_move(from,to,count);
#endif//KONTROL_B

#if SW_HISTORY
	//  
	list.sorting_score[count] =  History_heuristic::get_black_history(name,to); 
#endif//#if SW_HISTORY

	// 6  
	//   0
	list.move[count] = 0

	// 6  
	//    (0 -   1    2   3   
	//4    5    
	//12 -    13 -    14 -    15 -    
	//22 -      23 -      24 -      
	//25 -      )
	| (type_move << 6)

	// 6  
	//    (0 -63)
	| (from << 12)

	// 6  
	//    (0 -63)
	| (to << 18)

	// 3  
	//   (0 - 6)
	| (name << 24);

	//-------------------------------------------------------------------------------
	//std::cout << "move[p]= "<< move[p] << std::endl;
	//
	//std::cout << "0= "<< 0 << std::endl;
	//std::cout << "0= "<< ( move[p] & 63) << std::endl;
	//
	//std::cout << "d_m= "<< d_m << std::endl;
	//std::cout << "d_m= "<< ((move[p]>>6)& 63)  << std::endl;
	//
	//std::cout << "from= "<< from << std::endl;
	//std::cout << "from= "<< ((move[p]>>12)& 63)  << std::endl;
	//
	//std::cout << "to= "<< to << std::endl;
	//std::cout << "to= "<< ((move[p]>>18)& 63)  << std::endl;
	//
	//std::cout << "name= "<< name << std::endl;
	//std::cout << "name= "<<  ((move[p]>>24)& 7)  << std::endl;

	//-------------------------------------------------------------------------------

	count = count + 1;
}


//	===================================================================
//              
inline void Move_generation::list_record_special
(
	struct List & list,//   
	const uint8_t name,//   
	const uint8_t d_m,//  
	uint8_t & count,//    
	const uint8_t & from,//  
	const uint8_t to,//  
	const uint8_t capture//   
)
{
	//   +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

	//        
	//          
	list.sorting_score[count] = 110000;   //d_m;

	// 6  
	//   0
	list.move[count] = capture

	// 6  
	//    (0 -   1    2   3   
	//4    5    
	//12 -    13 -    14 -    15 -    
	//22 -      23 -      24 -      
	//25 -      )
	| (d_m << 6)

	// 6  
	//    (0 -63)
	| (from << 12)

	// 6  
	//    (0 -63)
	| (to << 18)

	// 3  
	//   (0 - 6)
	| (name << 24);

	//-------------------------------------------------------------------------------
	//std::cout << "move[p]= "<< move[p] << std::endl;
	//
	//std::cout << "0= "<< 0 << std::endl;
	//std::cout << "0= "<< ( move[p] & 63) << std::endl;
	//
	//std::cout << "d_m= "<< d_m << std::endl;
	//std::cout << "d_m= "<< ((move[p]>>6)& 63)  << std::endl;
	//
	//std::cout << "from= "<< from << std::endl;
	//std::cout << "from= "<< ((move[p]>>12)& 63)  << std::endl;
	//
	//std::cout << "to= "<< to << std::endl;
	//std::cout << "to= "<< ((move[p]>>18)& 63)  << std::endl;
	//
	//std::cout << "name= "<< name << std::endl;
	//std::cout << "name= "<<  ((move[p]>>24)& 7)  << std::endl;

	//-------------------------------------------------------------------------------

	count = count + 1;
}

//	===================================================================
//     ,         
//       
inline void Move_generation::KONTROL_captures_W
(
	const struct Position & position, //     
	const uint8_t & from,//   
	const uint8_t & to,//   
	const uint8_t & p,//    
	const uint8_t name_figure//   
)
{

#if KONTROL_B
	///////////////////////////////////////////////////////////////////////////////////
	if ((p < 0)||(p > 256))
	{
		std::cout << "W ERROR Generator_captures p= "   << p    << std::endl;
	}

	if ((from < 0)||(from > 63))
	{
		std::cout << "W ERROR Generator_captures from= " << from << std::endl;
	}

	if ((to < 0)||(to > 63))
	{
		std::cout << "W ERROR Generator_captures to= "  << to   << std::endl;
	}

	if ((position.white_name_from_coord[from] < 1)
	||(position.white_name_from_coord[from] > 6))
	{
		std::cout << "W ERROR Generator_captures position.white_name_from_coord[*from] = "  
			<< position.white_name_from_coord[from]<< std::endl;
	}

	if ((position.black_name_from_coord[to] < 1)
	||(position.black_name_from_coord[to] > 6))
	{
		std::cout << "W ERROR Generator_captures position.black_name_from_coord[*to]= " 
			<< position.black_name_from_coord[to]   << std::endl;
		std::cout << "W ERROR Generator_captures from= "<< from << std::endl;
		std::cout << "W ERROR Generator_captures to= "  << to   << std::endl;
		std::cout << "W ERROR Generator_captures name_figure= "  << name_figure << std::endl;
		position_print(position);
	}

	//if (position.black_name_from_coord[to] ==6)
	//{
		//std::cout << "ERROR Generator_captures name_figure KING "  << std::endl;
	//}
	///////////////////////////////////////////////////////////////////////////////////
#endif//KONTROL_B
}

//	===================================================================
//     ,         
//       
inline void Move_generation::KONTROL_captures_B
(
	const struct Position & position, //     
	const uint8_t & from,//   
	const uint8_t & to,//   
	const uint8_t & p,//    
	const uint8_t name_figure//   
)
{

#if KONTROL_B
	///////////////////////////////////////////////////////////////////////////////////
	if ((p < 0)||(p > 256))
	{
		std::cout << "B ERROR Generator_captures p= "   << p    << std::endl;
	}

	if ((from < 0)||(from > 63))
	{
		std::cout << "B ERROR Generator_captures from= " << from << std::endl;
	}

	if ((to < 0)||(to > 63))
	{
		std::cout << "B ERROR Generator_captures to= "  << to   << std::endl;
	}

	if ((position.black_name_from_coord[from] < 1)
	||(position.black_name_from_coord[from] > 6))
	{
		std::cout << "B ERROR Generator_captures position.black_name_from_coord[from] = "
			<< position.black_name_from_coord[from]   << std::endl;
	}

	if ((position.white_name_from_coord[to] < 1)
	||(position.white_name_from_coord[to] > 6))
	{
		std::cout << "B ERROR Generator_captures position.white_name_from_coord[to]= "
			<< position.white_name_from_coord[to]   << std::endl;
		std::cout << "B ERROR Generator_captures from= "<< from << std::endl;
		std::cout << "B ERROR Generator_captures to= "  << to   << std::endl;
		std::cout << "B ERROR Generator_captures name_figure= "  << name_figure << std::endl;
		position_print(position);
	}

	//if (position.white_name_from_coord[to] ==6)
	//{
	//std::cout << "B ERROR Generator_captures name_figure KING "  << std::endl;
	//}
///////////////////////////////////////////////////////////////////////////////////
#endif//KONTROL_B
}

//	===================================================================
//     ,    
//    
inline void Move_generation::KONTROL_move
(
	const uint8_t & from,//   
	const uint8_t & to,//   
	const uint8_t & p//    
)
{

#if KONTROL_B
	///////////////////////////////////////////////////////////////////////////////////
	if ((p < 0)||(p > 256))
	{
		std::cout << "ERROR Generator_move p= "   << p    << std::endl;
	}

	if ((from < 0)||(from > 63))
	{
		std::cout << "ERROR Generator_move from= " << from << std::endl;
	}

	if ((to < 0)||(to > 63))
	{
		std::cout << "ERROR Generator_move to= "  << to   << std::endl;
	}
	///////////////////////////////////////////////////////////////////////////////////
#endif//KONTROL_B
}

/*
       
 
 . -    Intel Pentium.
    (dos)     
     
(      :))
*/

//	===================================================================
//    (..  )
#include <assert.h>


const uint64_t magic = 0x03f79d71b4cb0a89;


const unsigned int magictable[64] =
    {
    0,  1, 48,  2, 57, 49, 28,  3,
    61, 58, 50, 42, 38, 29, 17,  4,
    62, 55, 59, 36, 53, 51, 43, 22,
    45, 39, 33, 30, 24, 18, 12,  5,
    63, 47, 56, 27, 60, 41, 37, 16,
    54, 35, 52, 21, 44, 32, 23, 11,
    46, 26, 40, 15, 34, 20, 31, 10,
    25, 14, 19,  9, 13,  8,  7,  6,
    };


int32_t Move_generation::bit_scan_forward_win32(const uint64_t bb){
assert (bb != 0);
return magictable[((bb&-bb)*magic) >> 58];
}


//	===================================================================
//    (..  )

int32_t Move_generation::bit_scan_reverse_win32(const uint64_t bb){

    union {
    double d;
    struct {
    unsigned int mantissal : 32;     
    unsigned int mantissah : 20;     
    unsigned int exponent : 11;     
    unsigned int sign : 1;     
    };
    } ud;
    ud.d = (double)(uint64_t)(bb & ~(bb >> 32));
    int idx = (ud.exponent - 1023) | (63*ud.sign);
    return idx;
    }

//	===================================================================
// 


/*

              &  0 0 1  
                  0 1 1  
                  -----
                  0 0 1 

            |  0 0 1
                  0 1 1
                  -----
                  0 1 1

    ^  0 0 1
               0 1 1
                  -----
                  0 1 0

*/
